from typing import Any, Dict, List, Union

DocumentType = Union[str, List[str]]
InstanceDict = Dict[str, Any]
MetricsType = Dict[str, Union[float, "MetricsType"]]
SummaryType = Union[str, List[str]]
TextType = Union[str, List[str]]
